/*
    Canopic Jars
    by G. Moran, 08 Nov 2021
    
    Licensed under CC BY-SA 4.0
    https://creativecommons.org/licenses/by-sa/4.0/
*/


#version 2.0

#include "material.inc"
#include "conical.inc"
#include "jbaboon.inc"
#include "jfalcon.inc"
#include "jhound.inc"
#include "jman.inc"
#include "windows.inc"
#include "sand.inc"
#include "coffin.inc"


/* LAYOUT
________________________________________*/


// viewport
camera {
    location <0, 0, 7>
    right <4/3, 0, 0>
    look_at <0,0,0>
    translate y*4.2
}

// lights
light_source { <-6, 5, 8> color White }

// dust
#declare Atmosphere = color Gold
fog {color Atmosphere distance 100}
background {color Atmosphere}


/* SCENE
________________________________________*/


// dimensions and location of sand floor
// width/depth : distance from y axis to x/z end of sand floor
// level : how far up the sand ends
#declare SandWidth = 9
#declare SandDepth = 4
#declare SandLevel = 4

// sand floor
object {
    Sand
    texture {LightSandy scale 1/500}
    normal {Wavy translate x*SandWidth -z*SandDepth scale 0.01}
    scale x*SandWidth + y*SandLevel + z*SandDepth
}

// submerged coffin
object {Coffin scale 2 translate y*SandLevel/2 translate -x/4}

// location of windows + jars
#declare WindowElev  = SandLevel-1
#declare WindowDepth = 2.5

// walls (right, left, up, front) with windows
union {
    //plane { x, -SandWidth}
    //plane {-x, -SandWidth}
    plane {-y, -8}
    difference {
        plane { z, -WindowDepth}
        object {FourWindows translate y*WindowElev + z*-WindowDepth}
    }
    texture {Sandy scale 1/100}
    texture {Cracked scale 6}
}

// canopic jars
object {Jars translate y*WindowElev + z*-WindowDepth}
