/* BABOON BUMPS
________________________________________*/


// single elongated bump sphere
// h = 2.2 (c. h = 4.5), w = 2
#declare BBump = sphere {
    y-y, 1
    scale (x+z)*2.0 + y*2.2
    translate x*1.2 + y*2.3
}

// baboon bumps
#declare BBumps = merge {
    object {BBump}
    object {BBump scale -x+y+z}
}


/* BABOON FACE
________________________________________*/


// upper half of sphere
// h = 4, r = 3
#declare BFace_ = difference {
    sphere {
        y-y, 1
        scale (x+z)*3 + y*4
    }
    plane {y, 0}
}

// baboon face + bumps
// h = 4.5, r = 3.2
#declare BFace = merge {
    object {BFace_}
    object {BBumps}
    bounded_by {
        box {<-3.2, 0, -3.2>, <3.2, 4.5, 3.2>}
    }
}


/* BABOON EYES
________________________________________*/


// eye elevation (vertical center)
#declare BEElev = 3.1
// eye offset in x and z
#declare BESide = 0.6
#declare BEOffset = 1.8

// fraction of upper eye half occupied by lid
#declare BEFrac = 1/2

// baboon eye upper half (long)
// -y plane : remove space for lid
// h = 0.5, w = 0.3
#declare BEye1 = difference {
    sphere {
        y-y, 1
        scale (x+z)*.3 + y/2
    }
    plane {y, 0}
    plane {-y, -BEFrac/2}
}

// baboon eye lower half (short)
// h = 0.3, w = 0.3
#declare BEye2 = difference {
    sphere {y-y, .3}
    plane {-y, 0}
}

// asymmetric baboon eye; long upper half, short lower half
// h = 0.8
#declare BEye = merge {
    object {BEye1}
    object {BEye2}
    translate x*BESide + y*BEElev
}

// baboon eyes
#declare BEyes = union {
    object {BEye}
    object {BEye scale -x+y+z}
    translate z*BEOffset
    no_shadow //
}


/* BABOON LIDS
________________________________________*/


// baboon eye lid
// fraction of upper eye half (long)
#declare BLid = difference {
    sphere {
        y-y, 1
        scale (x+z)*.3 + y/2
    }
    plane {y, BEFrac/2}
    translate x*BESide + y*BEElev
}

// baboon lids
#declare BLids = union {
    object {BLid}
    object {BLid scale -x+y+z}
    translate z*BEOffset
}


/* BABOON MOUF
________________________________________*/


// mouth elevation (vertical center)
#declare BMElev = 1.6

// major "mouf" sphere
// h = 2.8 (c. h = 3), r = 1.4
#declare BMouf_ = sphere {
    y-y, 1.4
    translate y*BMElev
}

// mouth opening
// h = 0.125, d = 1.5
#declare BSlit1 = box {
    <-2, 0, 1/2>, <2, 1/8, 2>
}

// one nose opening (out of two)
// 45  : slant angle
// -4/9, 3/4 : arbitrary values
// 1.1 : distance from mouth center to edge
// h = 0.4, w = 0.2
#declare BSlit2_ = sphere {
    y-y, 0.2
    scale <1/2, 1, 1>
    rotate z*45
    translate <-4/9, 3/4, 1.1>
}

// two nose openings
#declare BSlit2 = union {
    object {BSlit2_}
    object {BSlit2_ scale -x+y+z}
}

// baboon mouth + nose slits
// 25 : rotation angle to bring slits downwards
#declare BSlit = union {
    object {BSlit1}
    object {BSlit2}
    rotate x*25
    translate y*BMElev
    texture {m_BSlit}
}

// baboon "mouf"
// d = 4.4
#declare BMouf = difference {
    object {BMouf_}
    object {BSlit}
    bounded_by {
        sphere {y*BMElev, 1.4}
    }
    translate z*3
}


/* BABOON HEAD
________________________________________*/


// h = 1.5, r ~= 1.1
#declare BHead = union {
    object {BFace texture {m_BFace}}
    object {BEyes texture {m_BEyes}}
    object {BLids texture {m_BLids}}
    object {BMouf texture {m_BMouf}}
    
    bounded_by {
        box {<-3.2, 0, -3.2>, <3.2, 4.5, 4.4>}
    }
    scale 1/3
}


/* BABOON BODY
________________________________________*/


// baboon body
// h = 2, r = 1
#declare BBody = object {
    Body
    texture {m_BBody}
    bounded_by {
        cone {y-y, 1/3, y*2, 1}
    }
}


/* BABOON COLLAR (BALLS)
________________________________________*/


// single ball
// .25 : arbitrary value
#declare BBall = sphere {
    x*.75 + y*.15, .25
    translate y*2
}

// 8 (360/45) balls
#declare BBalls = union {
    object {BBall rotate y*0*45}
    object {BBall rotate y*1*45}
    object {BBall rotate y*2*45}
    object {BBall rotate y*3*45}
    object {BBall rotate y*4*45}
    object {BBall rotate y*5*45}
    object {BBall rotate y*6*45}
    object {BBall rotate y*7*45}
}

// baboon collar with ball holes
#declare BCollar1 = difference {
    object {Collar}
    object {BBalls}
}

// baboon collar balls
#declare BCollar2 = intersection {
    object {Collar}
    object {BBalls}
}

// baboon collar
// h = 0.3 (c. h = 2.3), r = 1
#declare BCollar = union {
    object {BCollar1 texture {m_BCollar1}}
    object {BCollar2 texture {m_BCollar2}}
    rotate y*22.5
    bounded_by {
        cone {y*2, 1, y*2.3, 1/2}
    }
}


/* BABOON
________________________________________*/


// baboon
// h ~= 3.1, r = 1
#declare Baboon = union {
    object {
        BHead
        scale 1/2
        translate y*2.3
    }
    object {BBody}
    object {BCollar}
    
    bounded_by {
        box {<-1, 0, -1>, <1, 3.1, 1>}
    }
}
