/* FALCON FACE
________________________________________*/


// control points of 2D bezier curve
// X
#declare X1 = 0
#declare X2 = 2
#declare X3 = 2
#declare X4 = 3
// Y
#declare Y1 = 4
#declare Y2 = 4
#declare Y3 = 3
#declare Y4 = 0

// bezier patch for half falcon face
// h = 4, r = 3
#declare Steps = 5
#declare FFace_ = object {
    #include "lathe2.inc"
}

// falcon face
#declare FFace = union {
    object {FFace_ rotate y*0}
    object {FFace_ rotate y*180}
}


/* FALCON SOCKETS
________________________________________*/


// submerged asymmetric falcon socket
// from two circle halves; short upper half and long lower half
// 20 : slant angle
// h = 1.5 or 0.5+1, w = 1
#declare FSocket_ = union {
    intersection {
        cylinder {
            z-z, z*4, 1
            scale x+z+y/2
        }
        plane {-y, 0 texture {None}}
    }
    intersection {
        cylinder {z-z, z*4, 1}
        plane { y, 0 texture {None}}
    }
    rotate z*20
}

// falcon socket
// .57 : arbitray value (trial and error)
// h ~= 0.9 or 0.3+0.6 (c. h ~= 3.3), w = 0.6
#declare FSocket = object {
    FSocket_
    scale (x+z)*3/5 + y*.57
    translate y*3
}

// falcon sockets
#declare FSockets = union {
    object {FSocket rotate y*30}
    object {FSocket scale -x+y+z rotate y*-30}
}


/* FALCON EYES
________________________________________*/


// falcon eye
// .15 : move eye down to actual vertical center of socket (0.6-0.3)/2
// .85 : approximate socket height
#declare FEye = sphere {
    y*(3-.15) + z*2, .85/2
}

// falcon eyes
#declare FEyes = union {
    object {FEye rotate y*-30}
    object {FEye rotate y* 30}
    no_shadow //
}


/* FALCON BEAK
________________________________________*/


// depth control points
#declare Z1 = 0
#declare Z2 = .5
#declare Z3 = 1
#declare Z4 = 1

// bezier patch for half falcon beak
// h = 1, d = 0.5
#declare Steps = 4
#declare FBeak_ = bicubic_patch {
    type 1
    flatness 0
    u_steps Steps
    v_steps Steps
    <0,1,Z1>, <.2,1,Z1>, <.2,.5,Z1>, <.5,.5,Z1>,
    <0,.9,Z2>, <.1,.9,Z2>, <.1,.5,Z2>, <.2,.5,Z2>,
    <0,.3,Z3>, <.02,.3,Z3>, <.05,.2,Z3>, <.05,.2,Z3>,
    <-0.01,0,Z4>, <-0.01,0,Z4>, <0.01,0,Z4>, <0.01,0,Z4>
    scale x+y+z/2
}

// falcon beak
// -15 : rotation angle to compensate for head curvature (trial and error)
// 1.4 : arbitrary value
// 2.4 : z offset value to complement rotation (trial and error)
// h = 1.6 (c. h = 3), d = 3.2
#declare FBeak = union {
    object {FBeak_}
    object {FBeak_ scale -x+y+z}
    rotate x*-15
    scale 1.6
    translate y*1.4 + z*2.4
}


/* FALCON HEAD
________________________________________*/


// falcon head
// h = 4/3, r ~= 1.1
#declare FHead = union {
    intersection {
        object {FSockets} object {FFace}
        texture {m_FSockets}
    }
    difference {
        object {FFace} object {FSockets}
        texture {m_FFace}
    }
    object {FEyes texture {m_FEyes}}
    object {FBeak texture {m_FBeak}}
    
    bounded_by {
        box {<-3,0,-3>, <3,4,3.2>}
    }
    scale 1/3
}


/* FALCON BODY
________________________________________*/


// falcon body
// h = 2, r = 1
#declare FBody = object {
    Body
    texture {m_FBody}
    bounded_by {
        cone {y-y, 1/3, y*2, 1}
    }
}


/* FALCON COLLAR (SPARKS)
________________________________________*/


// conical spark
// each covering a 45 deg arc of a circle with inner radius 0.5 and outer radius 1
// spark radius = inner collar radius * sin(45/2)
// spark offset = inner collar radius * cos(45/2)
// h = 0.3
#declare CRadiusI = .5
#declare CRadiusO = 1
#declare SRadius = CRadiusI * 0.38268
#declare SOffset = CRadiusI * 0.92388
#declare YI = 2.3
#declare YO = 2
#declare FSpark = cone {
    <SOffset,  YI, 0>, SRadius,
    <CRadiusO, YO, 0>, 0
}

// 8 (360/45) sparks
#declare FSparks = union {
    object {FSpark rotate y*0*45}
    object {FSpark rotate y*1*45}
    object {FSpark rotate y*2*45}
    object {FSpark rotate y*3*45}
    object {FSpark rotate y*4*45}
    object {FSpark rotate y*5*45}
    object {FSpark rotate y*6*45}
    object {FSpark rotate y*7*45}
}

// falcon collar with spark holes
#declare FCollar1 = difference {
    object {Collar}
    object {FSparks}
}

// falcon collar sparks
#declare FCollar2 = intersection {
    object {Collar}
    object {FSparks}
}

// falcon collar
// h = 0.3 (c. h = 2.3), r = 1
#declare FCollar = union {
    object {FCollar1 texture {m_FCollar1}}
    object {FCollar2 texture {m_FCollar2}}
    bounded_by {
        cone {y*2, 1, y*2.3, 1/2}
    }
}


/* FALCON 
________________________________________*/


// falcon
// h ~= 3, r = 1
#declare Falcon = union {
    object {
        FHead
        scale 1/2
        translate y*2.3
    }
    object {FBody}
    object {FCollar}
    
    bounded_by {
        box {-x-z, x+z+y*3}
    }
}
