/* HOUND SNOUT
________________________________________*/


// depth control points
#declare Z1 = 0
#declare Z2 = 1
#declare Z3 = 2.5
#declare Z4 = 3

// bezier patch for half hound snout
// h = 2 (c. h = 3), d = 3
#declare Steps = 4
#declare HSnout_ = bicubic_patch {
    type 1
    flatness 0
    u_steps Steps
    v_steps Steps
    <0,3  ,Z1>, <.4,3  ,Z1>, <2.5,1  ,Z1>, <0,1  ,Z1>,
    <0,2.4,Z2>, <.5,2.2,Z2>, < .8,1.6,Z2>, <0,1.6,Z2>,
    <0,2.4,Z3>, <.5,2.2,Z3>, < .8,1.6,Z3>, <0,1.6,Z3>,
    <0,2.4,Z4>, < 0,2.4,Z4>, < .8,2  ,Z4>, <0,2  ,Z4>
}

// hound snout end
#declare Steps = 5
#declare HEnd = bicubic_patch {
    type 1
    flatness 0
    u_steps Steps
    v_steps Steps
    <0,2.4,Z4>, < 0,2.4,Z4>, <-.8,2  ,Z4>, <0,2  ,Z4>
    <0,2.4,Z4>, < 0,2.4,Z4>, <0  ,2  ,Z4>, <0,2  ,Z4>
    <0,2.4,Z4>, < 0,2.4,Z4>, <0  ,2  ,Z4>, <0,2  ,Z4>
    <0,2.4,Z4>, < 0,2.4,Z4>, < .8,2  ,Z4>, <0,2  ,Z4>
}

// hound snout
// 2.65 : originally 3, z offset to move snout outside head (trial and error)
// 3.35 : scale to compensate for lost depth going from 3 to 2.65
// h = 2 (c. h = 3), d = 6
#declare HSnout = union {
    object {HSnout_}
    object {HSnout_ scale -x+y+z}
    object {HEnd}
    bounded_by {
        box {<-4/3,1,Z1>, <4/3,3,Z4>}
    }
    scale x+y+z*3.35/Z4
    translate z*2.65
}


/* HOUND FACE
________________________________________*/


// asymmetric hound face
// from two circle halves; short upper half and long lower half
// 2.9 : arbitrary value
// h = 3.1 or 1.2+1.9 (c. h = 4.1), w = 2.1
#declare HFace = merge {
    difference {
        quadric {
            <1,1,0>, <0,0,0>, <0,0,0>, -1
            scale x*2.1 + y*1.9 + z
        }
        plane {-y, 0}
        plane { z, 0}
    }
    difference {
        quadric {
            <1,1,0>, <0,0,0>, <0,0,0>, -1
            scale x*2.1 + y*1.2 + z
        }
        plane {y, 0}
        plane {z, 0}
    }
    translate y*2.9
    bounded_by {
        box {<-2.1, 1, 2>, <2.1, 4.1, 3>}
    }
}


/* HOUND EARS
________________________________________*/


// one ear
// from a flat bezier patch
// h = 6 (c. h = 10)
#declare Z = 0
#declare HEar = bicubic_patch {
    type 1
    flatness 0
    u_steps 0
    v_steps 0
    <1.5,3.7,Z>, -z, -z, <1.5,3.7,Z>,
    -z, -z, -z, -z,
    -z, -z, -z, -z,
    <3  ,5.6,Z>, -z, -z, <1.5,10 ,Z>
}

// hound ears
#declare HEars = union {
    object {HEar}
    object {HEar scale -x+y+z}
}


/* HOUND SOCKETS
________________________________________*/


// hound socket
// from half circle
// 3.3 : arbitrary value
// h = 0.4 (c. h = 3.3), w = 0.5
#declare HSocket_ = difference {
    quadric {
        <1,1,0>, <0,0,0>, <0,0,0>, -1
        scale z + x*.5 + y*.4
    }
    plane {-y, 0}
    translate y*3.3
}

// hound sockets
#declare HSockets = union {
    object {HSocket_ rotate y* 24}
    object {HSocket_ rotate y*-24}
    bounded_by {
        box {<-2, 2.9, 2>, <2, 3.3, 3>}
    }
}


/* HOUND EYES
________________________________________*/


// elongated hound eye vertically centered within socket
// 2.6 : z offset to move eye outside head
// h = 0.4, w = 0.3
#declare HEye = sphere {
    y-y, 1
    scale y*.2 + (x+z)*.15
    translate y*3.1
    translate z*2.6
}

// hound eyes
#declare HEyes = union {
    object {HEye rotate y* 24}
    object {HEye rotate y*-24}
    no_shadow //
}


/* HOUND NECK
________________________________________*/


// control points of 2D bezier curve
// X
#declare X1 = 0
#declare X2 = 3
#declare X3 = 3
#declare X4 = 3
// Y
#declare Y1 = 4.5
#declare Y2 = 4.5
#declare Y3 = 2
#declare Y4 = 0

// bezier patch for half hound neck
// h = 4.5, r = 3
#declare Steps = 5
#declare HNeck_ = object {
    #include "lathe2.inc"
}

// hound neck
#declare HNeck = union {
    object {HNeck_}
    object {HNeck_ rotate y*180}
    bounded_by {
        box {<-3,0,-3>, <3,4.5,3>}
    }
}


/* HOUND HEAD
________________________________________*/


// hound head
// h ~= 3.4, r = 1
#declare HHead = union {
    difference {
        object {HNeck} object {HFace}
        texture {m_HNeck}
    }
    difference {
        intersection {
            object {HNeck} object {HFace}
        }
        object {HSockets}
        texture {m_HFace}
    }
    intersection {
        object {HNeck} object {HSockets}
        texture {m_HSockets}
    }
    object {HSnout texture {m_HFace}}
    object {HEars  texture {m_HEars}}
    object {HEyes  texture {m_HEyes}}
    
    bounded_by {
        box {<-3,0,-3>, <3,10,6>}
    }
    scale 1/3
}


/* HOUND BODY
________________________________________*/


// hound body
// h = 2, r = 1
#declare HBody = object {
    Body
    texture {m_HBody}
    bounded_by {
        cone {y-y, 1/3, y*2, 1}
    }
}


/* HOUND COLLAR (RING)
________________________________________*/


// hound ring vertically centered in collar
// h = 0.15
#declare HRing = cone {
    y*2.075, 1, y*2.225, 1
}

// hound collar with a ring hole
#declare HCollar1 = difference {
    object {Collar}
    object {HRing}
}

// hound collar ring
#declare HCollar2 = intersection {
    object {Collar}
    object {HRing}
}

// hound collar
// h = 0.3 (c. h = 2.3), r = 1
#declare HCollar = union {
    object {HCollar1 texture {m_HCollar1}}
    object {HCollar2 texture {m_HCollar2}}
    bounded_by {
        cone {y*2, 1, y*2.3, 1/2}
    }
}


/* HOUND
________________________________________*/


// hound
// h ~= 4, r = 1
#declare Hound = union {
    object {
        HHead
        scale 1/2
        translate y*2.3
    }
    object {HBody}
    object {HCollar}
    
    bounded_by {
        box {<-1, 0, -1>, <1, 4, 1>}
    }
}
