/* MAN MANE
________________________________________*/


// control points of 2D bezier curve
// X
#declare X1 = 0
#declare X2 = 1
#declare X3 = 3.5
#declare X4 = 3.5
// Y
#declare Y1 = 5.5
#declare Y2 = 5.5
#declare Y3 = 5.5
#declare Y4 = 1

// bezier patch for half of upper man mane
// h = 4.5 (c. h = 5.5), r = 3.5
#declare Steps = 5
#declare MMane1_ = object {
    #include "lathe2.inc"
}

// upper man mane
// z/2 : half depth in +ve z
#declare MMane1 = union {
    object {MMane1_ rotate y*0 scale x+y+z/2}
    object {MMane1_ rotate y*180}
}

// control points of 2D bezier curve
// X
#declare X1 = 3.5
#declare X2 = 3.5
#declare X3 = 3
#declare X4 = 3
// Y
#declare Y1 = 1
#declare Y2 = .8
#declare Y3 = 1
#declare Y4 = 0

// bezier patch for half of lower man mane
// h = 1, r = 3
#declare Steps = 5
#declare MMane2_ = object {
    #include "lathe2.inc"
}

// lower man mane
// z/2 : half depth in +ve z
#declare MMane2 = union {
    object {MMane2_ rotate y*0 scale x+y+z/2}
    object {MMane2_ rotate y*180}
}

// man mane
// h = 5.5, r = 3.5, -3.5 <= z <= 1.75
#declare MMane = union {
    object {MMane1} object {MMane2}
    bounded_by {
        box {<-3.5,0,-3.5>, <3.5,5.5,1.75>}
    }
}


/* MAN FACE
________________________________________*/


// ding dong surface (trimmed)
// h = 1, r max = 0.3849 or sqrt(4/27)
#declare DingDong = intersection {
    cubic {<
         0,  0,  0,  1,
         0,  0,  0,  0,
         0,  0,  1,  0,
        -1,  0,  0,  0,
         0,  1,  0,  0
    > sturm}
    sphere {y*5/9, 5/9 texture {None}}
    bounded_by {
        sphere {y/2, 1/2}
    }
}

// man face
// h = 4.5
#declare MFace = object {
    DingDong
    scale 4.5
}


/* MAN CHIN
________________________________________*/


// man chin (mostly underground)
// h = 1.2
#declare MChin = union {
    cone {y-y, .3, -y, .2}
    sphere {-y, .2}
    bounded_by {
        box {<-.3,-1.2,-.3>, <.3,0,.3>}
    }
    translate y*.3
}


/* MAN SOCKETS
________________________________________*/


// submerged symmetric man socket
// from intersection of two identical circles
// h = 1, w ~= 0.9 * 2
#declare MSocket_ = intersection {
    cylinder {<0, .5,0>, <0, .5,2>, 1}
    cylinder {<0,-.5,0>, <0,-.5,2>, 1}
    bounded_by {
        //cylinder {<0,0,0>, <0,0,2>, 1 scale <.9,.5,1>}
        box {<-.9,-.5,0>, <.9,.5,2>}
    }
}

// man socket
// 2.5 : arbitrary value (trial and error)
// -15 : slant socket slightly
// h = 0.5, w ~= 1.1 or 0.54*2
#declare MSocket = object {
    MSocket_
    scale <.6,.5,1>
    rotate z*-15 //
    translate y*2.5
}

// man sockets
#declare MSockets = union {
    object {MSocket rotate y*-30}
    object {MSocket rotate y*-30 scale -x+y+z}
}


/* MAN EYES
________________________________________*/


// man eye centered in socket
// 2.5  : arbitrary value (trial and error, same as socket)
// 1.65 : arbitrary value (trial and error)
#declare MEye = sphere {
    y-y, 1/4
    translate y*2.5
    translate z*1.65
}

// man eyes
#declare MEyes = union {
    object {MEye rotate y*-30}
    object {MEye rotate y* 30}
    no_shadow //
}


/* MAN EYEBROWS
________________________________________*/


// intersection of socket with squashed circle
// h = 0.5, thickness = 0.1
#declare MEyebrow_ = difference {
    object {MSocket_}
    cylinder {
        <0,0,0>, <0,0,2>, 1
        scale x+z+y*.8
        translate y*-.4
    }
    bounded_by {
        box {<-.9,0,0>, <.9,.5,2>}
    }
}

// man eyebrow
// -15 : slant eyebrow slightly (same as socket)
// h = 0.25, g < 0.1
#declare MEyebrow = object {
    MEyebrow_
    scale <.6,.5,1>
    rotate z*-15 //
    translate y*2.7
}

// man eyebrows
#declare MEyebrows = union {
    object {MEyebrow rotate y*-30}
    object {MEyebrow rotate y*-30 scale -x+y+z}
}


/* MAN NOSE
________________________________________*/


// man nose
// 1.3 : arbitrary value (trial and error)
#declare MNose = cone {
    y-y, .3, y*1.5, 0
    translate y*1.3
    translate z*1.5
}


/* MAN MOUTH
________________________________________*/


// man mouth
// .2 : arbitrary value (trial and error)
#declare MMouth = difference {
    cylinder {<0,0,.2>, <0,0,2>, .5}
    cylinder {<0,0,.2>, <0,0,2>, 1
        scale <.7,.5,1>
        translate y*.2
    }
    
    bounded_by {
        box {<-.5,-.5,0>, <.5,-.2,2>}
    }
    translate y*1.3
}


/* MAN HEAD
________________________________________*/


// man head
// -30 : tilt face backwards
// h ~= 2, r ~= 1.2
#declare MHead = union {
    union {
        difference {
            object {MFace}
            object {MEyebrows} object {MSockets}
            object {MMouth}
            texture {m_MFace}
        }
        intersection {
            object {MFace} object {MSockets}
            texture {m_MSockets}
        }
        intersection {
            object {MFace} object {MEyebrows}
            texture {m_MSockets}
        }
        object {MEyes texture {m_MEyes}}
        object {MNose texture {m_MFace}}
        object {MChin texture {m_MChin}}
        //
        translate y*-2.25
        rotate x*-30
        translate y* 2.25
        //
        translate y*.5
        translate z
    }
    object {MMane texture {m_MMane}}
    
    bounded_by {
        box {<-3.5,0,-3.5>, <3.5,5.5,3.5>}
    }
    scale 1/3
}


/* MAN BODY
________________________________________*/


// man body
// h = 2, r = 1
#declare MBody = object {
    Body
    texture {m_MBody}
    bounded_by {
        cone {y-y, 1/3, y*2, 1}
    }
}


/* MAN COLLAR (STRIPES)
________________________________________*/


// conical stripe
// 0.2 : arbitrary value for stripe radius
// 2.6 : collar height if it was a complete cone
// h = 0.3
#declare MStripe = cone {
    <0, 2.6, 0>, 0,
    <1, 2  , 0>, .2
}

// 8 (360/45) stripes
#declare MStripes = union {
    object {MStripe rotate y*0*45}
    object {MStripe rotate y*1*45}
    object {MStripe rotate y*2*45}
    object {MStripe rotate y*3*45}
    object {MStripe rotate y*4*45}
    object {MStripe rotate y*5*45}
    object {MStripe rotate y*6*45}
    object {MStripe rotate y*7*45}
}

// man collar with stripe holes
#declare MCollar1 = difference {
    object {Collar}
    object {MStripes}
}

// man collar stripes
#declare MCollar2 = intersection {
    object {Collar}
    object {MStripes}
}

// man collar
// h = 0.3 (c. h = 2.3), r = 1
#declare MCollar = union {
    object {MCollar1 texture {m_MCollar1}}
    object {MCollar2 texture {m_MCollar2}}
    rotate y*22.5
    bounded_by {
        cone {y*2, 1, y*2.3, 1/2}
    }
}


/* MAN
________________________________________*/


// man
// h ~= 3.3, r = 1
#declare Man = union {
    object {
        MHead
        scale 1/2
        translate y*2.3
    }
    object {MBody}
    object {MCollar}
    
    bounded_by {
        box {-x-z, x+z+y*3.3}
    }
}
