#include "colors.inc"


/* INIT TEXTURES
________________________________________*/


// set default ambient and diffuse
#default {
    finish {
        diffuse 0.70
        ambient 0.25
    }
}

// invisible texture
#declare None = texture {
    pigment {
        color Clear
        quick_color Black
    }
    finish {ambient 1 diffuse 0}
}

// debug texture
#declare Rad = texture {
    pigment {color Red filter 0.7}
    finish {phong 1}
}


/* GLOBAL TEXTURES
________________________________________*/


// shiny gold
#declare Golden = texture {
    pigment {color Orange}
    finish  {phong .5 phong_size 1}
}

// shinier gold
#declare GoldenXtra = texture {
    pigment {color Orange}
    finish  {phong 1 phong_size 1 diffuse 4/3}
}

// lapis lazuli (plain)
#declare Lapis_ = texture {
    pigment {color Navy}
    finish  {phong 1 phong_size 40}
}

// lapis lazuli
#declare Lapis = texture {
    pigment {
        granite
        color_map {
            [0.0 color MidnightBlue]
            [0.5 color Navy]
            [1.0 color RichBlue]
        }
        frequency -1
        quick_color Navy
    }
    finish  {phong 1 phong_size 40}
}

// carnelian
#declare Carnelian = texture {
    pigment {color rgb <.8, 0, 0>}
    finish  {phong 1 phong_size 80 diffuse .8}
}

// clay (plain)
#declare Clay_ = texture {
    pigment {color Firebrick}
    finish  {phong 1 phong_size 40}
}

// clay
#declare Clay = texture {
    pigment {color Firebrick}
}
texture {
    pigment {
        bozo
        turbulence 2
        lambda 4
        color_map {
            [0.0 color Firebrick filter 0.7] [0.6 color Firebrick filter 0.7]
            [0.8 color Black filter 0.7]
            [1.0 color White filter 0.7]
        }
        rotate z*-45
        quick_color Firebrick
    }
    finish  {phong 1 phong_size 40}
}

// fair sandy texture (plain)
#declare Sandy_ = texture {
    pigment {color Gold}
    finish  {phong 0}
}

// fair sandy texture
#declare Sandy = texture {
    pigment {
        bozo
        color_map {
            [0.0 color Copper]
            [0.5 color Gold]
            [1.0 color Tan]
        }
        quick_color Gold
    }
    finish  {phong 0}
}

// light sandy texture
#declare LightSandy = texture {
    pigment {
        bozo
        color_map {
            [0.0 color CoolCopper]
            [0.5 color Gold]
            [1.0 color MediumGoldenrod]
        }
        quick_color MediumGoldenrod
    }
    finish  {phong 0}
}


/* GLOBAL OVERLAYS
________________________________________*/


// wavy bumps (for sandy textures)
#declare Wavy = normal {
    ripples .4 frequency .05
}

// cracks (for walls)
#declare Cracked = texture {
    pigment {
        agate agate_turb 2
        color_map {
            [0.00 color White filter 1.0] [0.99 color White filter 1.0]
            [0.99 color Black filter 0.8] [1.00 color Black filter 0.8]
        }
        rotate z*45
    }
}

// dented texture (for coffin)
#declare Dented = normal {
    dents .2 scale .015
}


/* OBJECT-ASSIGNED TEXTURES
________________________________________*/


// falcon
#declare m_FFace    = texture {Clay  scale 6 translate y*100}
#declare m_FSockets = texture {GoldenXtra}
#declare m_FEyes    = texture {Lapis scale 6 translate y*100}
#declare m_FBeak    = texture {GoldenXtra}
#declare m_FBody    = texture {Clay          translate y*100}
#declare m_FCollar1 = texture {GoldenXtra}
#declare m_FCollar2 = texture {Lapis         translate y*100}

// baboon
#declare m_BFace    = texture {Lapis scale 6 translate y*200}
#declare m_BEyes    = texture {Carnelian}
#declare m_BLids    = texture {GoldenXtra}
#declare m_BSlit    = texture {Carnelian}
#declare m_BMouf    = texture {Golden}
#declare m_BBody    = texture {Lapis         translate y*200}
#declare m_BCollar1 = texture {Carnelian}
#declare m_BCollar2 = texture {GoldenXtra}

// hound
#declare m_HFace    = texture {Lapis scale 6 translate y*300}
#declare m_HEars    = texture {Lapis scale 6 translate y*300}
#declare m_HSockets = texture {GoldenXtra}
#declare m_HEyes    = texture {Carnelian}
#declare m_HNeck    = texture {Golden}
#declare m_HBody    = texture {Lapis         translate y*300}
#declare m_HCollar1 = texture {GoldenXtra}
#declare m_HCollar2 = texture {Carnelian}

// man
#declare m_MFace    = texture {Clay  scale 6 translate y*400}
#declare m_MChin    = texture {Lapis scale 6 translate y*400}
#declare m_MSockets = texture {GoldenXtra}
#declare m_MEyes    = texture {Lapis scale 6 translate y*400}
#declare m_MMane    = texture {Golden}
#declare m_MBody    = texture {Clay          translate y*400}
#declare m_MCollar1 = texture {GoldenXtra}
#declare m_MCollar2 = texture {Lapis         translate y*400}

// coffin
#declare m_Coffin   = texture {Golden normal {Dented}}
