/* WINDOW
________________________________________*/


// window in wall, centered about y axis
// z*2 : double depth
// total (H)eight, (R)adius), and (G)ap are user defined
#declare H = 4.2
#declare R = 1.2
#declare G = R/3
#declare Window = merge {
    cylinder {y-y, y*(H-R), R}
    sphere {y*(H-R), R}
    bounded_by {
        box { <-R,0,-R>, <R,H,R> }
    }
    scale x+y+z*2
}


/* FOUR WINDOWS
________________________________________*/


// four windows, centered about y axis
// first positioned along -ve x then centered
// 0 <= x <= -(n 2R + (n-1) G)
#declare FourWindows = union {
    object {Window translate -x*R}
    object {Window translate -x*R -x*(2*R+G)*1}
    object {Window translate -x*R -x*(2*R+G)*2}
    object {Window translate -x*R -x*(2*R+G)*3}
    
    translate x*(4*2*R + 3*G) / 2
}


/* CENTERED CANOPIC JARS
________________________________________*/


// canopic jars centered within windows
// h ~= 4, r = 1 * 4
#declare Jars = union {
    object {Baboon translate -x*R}
    object {Falcon translate -x*R -x*(2*R+G)*1}
    object {Man    translate -x*R -x*(2*R+G)*2}
    object {Hound  translate -x*R -x*(2*R+G)*3}
    
    translate -z
    translate x*(4*2*R + 3*G) / 2
}
