/* GHOST BODY
________________________________________*/


// sphere and cylinder
// h = 7, r = 3
#declare GBody_ = merge {
    sphere {y*4, 3}
    cylinder {y*.0001, y*4, 3}
}

// extra cylinder
#declare GHollow = cylinder {
    y-y, y*4, 2.99
}

// bottom spike
// each covering a 45 deg arc of a circle with radius 3
// h = 1.5, r = 1.2426 or 3*tan(22.5)
#declare GSpike = cone {
    x*3, 1.2426, x*3+y*1.5, 0
}

// 8 (360/45) bottom spikes
#declare GSpikes = union {
    object {GSpike rotate y*45*0}
    object {GSpike rotate y*45*1}
    object {GSpike rotate y*45*2}
    object {GSpike rotate y*45*3}
    object {GSpike rotate y*45*4}
    object {GSpike rotate y*45*5}
    object {GSpike rotate y*45*6}
    object {GSpike rotate y*45*7}
}

// ghost body
#declare GBody = difference {
    object {GBody_} object {GHollow}
    object {GSpikes}
}


/* GHOST EYES
________________________________________*/


// ghost eyes rotation angle
#declare GEAngle = 20

// ghost eye ball
// z/2 : half depth
// 2.8 : move eyes outside body
// c. h = 5, r = 1
#declare GEye = sphere {
    y-y, 1
    scale x+y+z/2
    translate y*4
    translate z*2.8
    no_shadow
}

// ghost eye balls
#declare GEyes = union {
    object {GEye rotate y*-GEAngle}
    object {GEye rotate y* GEAngle}
}


/* GHOST PUPILS
________________________________________*/


// ghost pupil
// c. h = 4.5, r = 0.5
#declare GPupil = cylinder {
    z*2, z*4, .5
    translate y*4
    no_shadow
}

// blinky pupils (down right)
// 7, 7 : arbitrary values
#declare GPBlinky = union {
    object {GPupil rotate y*-7+x*7 rotate y*-GEAngle}
    object {GPupil rotate y*-7+x*7 rotate y* GEAngle}
}

// inky pupils (down left)
// 7, 7 : arbitrary values
#declare GPInky = union {
    object {GPupil rotate y*7+x*7 rotate y*-GEAngle}
    object {GPupil rotate y*7+x*7 rotate y* GEAngle}
}


/* GHOSTS
________________________________________*/


// blinky (red)
#declare Blinky = union {
    object {GBody texture {m_Blinky}}
    difference {
        object {GEyes} object {GPBlinky}
        texture {m_GEye}
    }
    intersection {
        object {GEyes} object {GPBlinky}
        texture {m_GPupil}
    }
    bounded_by {
        box {<-3, 0, -3>, <3, 7, 3.3>}
    }
    scale 1/3
}

// inky (cyan)
#declare Inky = union {
    object {GBody texture {m_Inky}}
    difference {
        object {GEyes} object {GPInky}
        texture {m_GEye}
    }
    intersection {
        object {GEyes} object {GPInky}
        texture {m_GPupil}
    }
    bounded_by {
        box {<-3, 0, -3>, <3, 7, 3.3>}
    }
    scale 1/3
}

// clyde (orange)
#declare Clyde = union {
    object {GBody texture {m_Clyde}}
    difference {
        object {GEye} object {GPupil}
        texture {m_GEye}
    }
    intersection {
        object {GEye} object {GPupil}
        texture {m_GPupil}
    }
    bounded_by {
        box {<-3, 0, -3>, <3, 7, 3.3>}
    }
    scale 1/3
}
