/*
    Maze walls
    block width and cap diameter = 0.6
    N = -z, E = -x
*/


/* TIER 1 PARTS
________________________________________*/


// round cap
#declare Cap = cylinder {
    y-y, y, .3
}

// vertical block
// 0 <= z <= 1, centered about z
#declare BlockV = box {
    <-.3, 0, 0>, <.3, 1, 1>
}

// horizontal block
// 0 <= x <= 1, centered about x
#declare BlockH = box {
    <0, 0, -.3>, <1, 1, .3>
}

// inner curve
// 0.3 x 0.3 block
#declare Innie = difference {
    box      {<.3,  0, .3>, <.6, 1, .6>}
    cylinder {<.6, -1, .6>, <.6, 2, .6>, .3}
}

// outer curve
// 0.6 x 0.6 block
#declare Outie = intersection {
    box      {<-.3, 0, -.3>, <.3, 1, .3>}
    cylinder {<.3, -1,  .3>, <.3, 2, .3>, .6}
}


/* TIER 2 PARTS
________________________________________*/


// 4 caps
#declare CapN = intersection {
    object {Cap} plane { z,0}
}
#declare CapS = intersection {
    object {Cap} plane {-z,0}
}
#declare CapE = intersection {
    object {Cap} plane { x,0}
}
#declare CapW = intersection {
    object {Cap} plane {-x,0}
}

// 4 inner curves
#declare InnieSW = object {Innie}
#declare InnieSE = object {Innie scale -x+y+z}
#declare InnieNE = object {Innie scale -x+y-z}
#declare InnieNW = object {Innie scale  x+y-z}

// 4 outer curves
#declare OutieSW = object {Outie}
#declare OutieSE = object {Outie scale -x+y+z}
#declare OutieNE = object {Outie scale -x+y-z}
#declare OutieNW = object {Outie scale  x+y-z}


/* TIER 3 PARTS
________________________________________*/


// corners
#declare CornerSW = merge {
    object {OutieSW} object {InnieSW}
}
#declare CornerSE = merge {
    object {OutieSE} object {InnieSE}
}
#declare CornerNE = merge {
    object {OutieNE} object {InnieNE}
}
#declare CornerNW = merge {
    object {OutieNW} object {InnieNW}
}

// three way intersections
#declare IntN = merge {
    object {InnieNE} object {InnieNW}
}
#declare IntS = merge {
    object {InnieSE} object {InnieSW}
}
#declare IntE = merge {
    object {InnieNE} object {InnieSE}
}
#declare IntW = merge {
    object {InnieNW} object {InnieSW}
}


/* MAZE WALLS
________________________________________*/


// wall 1 (vertical)
#declare Wall1 = merge {
    object {CapN}
    object {BlockV scale <1,1,2>}
    object {CapS translate <0,0,2>}
    scale x+z+y*2
    bounded_by {
        box {<-.3, 0, -.3>, <.3, 2, 2.3>}
    }
    texture {m_Walls}
    no_shadow
}

// wall 2 (horizontal)
#declare Wall2 = merge {
    object {CapE}
    object {BlockH scale <3,1,1>}
    object {CapW translate <3,0,0>}
    scale x+z+y*2
    bounded_by {
        box {<-.3, 0, -.3>, <3.3, 2, .3>}
    }
    texture {m_Walls}
    no_shadow
}

// wall 3 (three way)
#declare Wall3 = merge {
    object {CapE}
    object {BlockH scale <4,1,1>}
    object {CapW translate <4,0,0>}
    object {BlockV scale <1,1,2-.3> translate <2,0,.3>}
    object {CapS translate <2,0,2>}
    object {IntS translate <2,0,0>}
    scale x+z+y*2
    bounded_by {
        union {
            box {<-.3, 0, -.3>, <4.3, 2,  .3>}
            box {<1.7, 0,  .3>, <2.3, 2, 2.3>}
            box {<1.4, 0, .3>, <2.6, 2, .6>}
        }
    }
    texture {m_Walls}
    no_shadow
}

// wall 4 (vertical)
#declare Wall4 = object {Wall2 rotate y*-90}
