#include "colors.inc"


/* INIT TEXTURES
________________________________________*/


// set default ambient and diffuse
#default {
    finish {
        diffuse .7
        ambient .3
    }
}

// invisible texture
#declare None = texture {
    pigment {
        color Clear
        quick_color Black
    }
    finish {ambient 1 diffuse 0}
}

// debug texture
#declare Rad = texture {
    pigment {color Yellow filter 0.7}
    finish {phong 1}
}


/* SOLID TEXTURES
________________________________________*/


// red
#declare Reddish = texture {
    pigment {color Red}
    finish  {phong 1 phong_size 40}
}

// dark red
#declare DarkReddish = texture {
    pigment {color rgb <.85,0,0>}
    finish  {phong 1 phong_size 40}
}

// orangish yellow
#declare Orangish = texture {
    pigment {color rgb <1,.75,0>}
    finish  {phong 1 phong_size 40}
}

// black
#declare TheAbyss = texture {
    pigment {color Black}
    finish  {phong 0}
}


/* DULL TEXTURES
________________________________________*/


// deep blue feathers
#declare DeepBlue = texture {
    pigment {color NewMidnightBlue}
    finish  {phong .25 phong_size 10}
}

// white feathers
#declare Feathery = texture {
    pigment {color Silver}
    finish  {phong .25 phong_size 10}
}


/* SHINY TEXTURES
________________________________________*/


// shining white
#declare Shining = texture {
    pigment {color White}
    finish  {phong 1 phong_size 80}
}

// brighter shining white
#declare ShiningXtra = texture {
    Shining
    finish {ambient .4}
}

// shiny dark gray
#declare ShinyGrayish = texture {
    pigment {color Gray25}
    finish  {phong 1 phong_size 2}
}

// shiny silver
#declare ShinySilver = texture {
    pigment {color Gray85}
    finish  {phong 1 phong_size 2}
}

// shiny gold
#declare Golden = texture {
    pigment {color Orange}
    finish  {phong .5 phong_size 1}
}


/* GLASS TEXTURES
________________________________________*/


// transparent glass
#declare Glass = texture {
    pigment {color White filter .9 quick_color Turquoise}
    finish {
        phong .5 phong_size 40
        refraction 1 ior 1.025
    }
}

// light reflective glass
#declare Reflective = texture {
    pigment {color White quick_color White}
    finish {
        reflection .25
        ambient .2
    }
}

// dark reflective glass
#declare ReflectiveD = texture {
    pigment {color Gray60 quick_color Gray60}
    finish {
        reflection .25
        ambient .2
    }
}

// shiny red overlay
#declare OverlayR = texture {
    pigment {color Red filter 1 quick_color Red}
    finish {phong 1 phong_size 40}
}

// shiny blue overlay
#declare OverlayB = texture {
    pigment {color Blue filter 1 quick_color Blue}
    finish {phong 1 phong_size 40}
}


/* PATTERN TEXTURES
________________________________________*/


// snow texture
#declare Snowy = texture {
    pigment {color Silver}
    normal {bumps .75}
    finish {
        phong 1 phong_size 2
        ambient .4
    }
}

// falling snow
#declare Snowfall = texture {
    pigment {
        leopard turbulence .5 scale 1/16
        color_map {
            [.75 color White filter 1]
            [.75 color White filter 0]
        }
        quick_color Silver
    }
    finish {ambient .8 diffuse .2}
}
texture {
    pigment {
        bozo scale 1/8
        color_map {
            [.99 color White filter 1]
            [.99 color White filter 0]
        }
    }
    finish {ambient .8 diffuse .2}
}
texture {
    pigment {
        bozo scale 1/8
        color_map {
            [.999 color White filter 1]
            [.999 color White filter 0]
        }
        translate x*1000
    }
    finish {ambient .8 diffuse .2}
}

// green stripes
#declare Striped = texture {
    pigment {color rgb <.1,.3,.1>}
    finish {ambient 1 diffuse .5}
}
texture {
    pigment {
        gradient x scale 3 translate x*30
        color_map {
            [.5 color Black filter .85]
            [.5 color White filter 1]
        }
        quick_color DarkGreen
    }
}

// wooden texture
#declare Wooden = texture {
    pigment {
        wood rotate y*90
        turbulence .05
        octaves 5
        color_map {
            [.1 color Feldspar]
            [.9 color DarkWood]
        }
        quick_color DarkWood
    }
}


/* OBJECT-ASSIGNED TEXTURES
________________________________________*/


// penguin
#declare m_PEyes1   = texture {TheAbyss}
#declare m_PEyes2   = texture {ShiningXtra}
#declare m_PBeak    = texture {Orangish}
#declare m_PHat1    = texture {DarkReddish}
#declare m_PHat2    = texture {Shining}
#declare m_PBody1   = texture {DeepBlue}
#declare m_PBody2   = texture {Feathery}
#declare m_PMouth   = texture {DarkReddish}
#declare m_PTalons  = texture {Orangish}

// snow globe
#declare m_SGlobe   = texture {Glass}
#declare m_SBase1   = texture {ShinyGrayish}
#declare m_SBase2   = texture {ShinySilver}
#declare m_SFloor   = texture {Snowy}
#declare m_SFall    = texture {Snowfall}

// xmas balls
#declare m_XBall    = texture {Reflective}
#declare m_XBallD   = texture {ReflectiveD}
#declare m_XShellR  = texture {OverlayR}
#declare m_XShellB  = texture {OverlayB}
#declare m_XTip     = texture {Golden}
#declare m_XHook    = texture {Golden}
