/* PENGUIN EYES
________________________________________*/


// penguin eye with twinkly gap
// h = 0.8, w = 0.4
#declare PEye1 = difference {
    cylinder {
        z-z, z*2, 1
    }
    cylinder {
        x/2+y/2, x/2+y/2+z*2, .5
    }
    scale <.2, .4, 1>
    translate y*.5
}

// penguin eye twinkle
#declare PEye2 = intersection {
    cylinder {
        z-z, z*2, 1
    }
    cylinder {
        x/2+y/2, x/2+y/2+z*2, .5
    }
    scale <.2, .4, 1>
    translate y*.5
}

// penguin eyes with twinkly gap
#declare PEyes1 = union {
    object {PEye1 rotate y*-22}
    object {PEye1 rotate y* 22}
}

// penguin eye twinkles
#declare PEyes2 = union {
    object {PEye2 rotate y*-22}
    object {PEye2 rotate y* 22}
}


/* PENGUIN BEAK
________________________________________*/


// parabola surface (trimmed)
// h = 1, r = 1
#declare Parabola = difference {
    quadric {
        <-1,0,-1>, <0,0,0>, <0,-1,0>, 1
        inverse
    }
    plane {y, 0 texture {None}}
}

// horizontal parabola
#declare Parabola_ = object {
    Parabola
    rotate x*90
}

// upper beak half
// h = 0.5, d = 1.2
#declare PBeakU = difference {
    object {Parabola_}
    object {Parabola_ scale .75}
    plane  { y, 0}
    
    bounded_by {
        box {<-1, 0, 0>, <1, 1, 1>}
    }
    scale <1, 1, 1.2>
}

// lower beak half (rotated)
// h = 0.25, d = 1.2
#declare PBeakD = difference {
    object {Parabola_}
    object {Parabola_ scale .75 texture {m_PMouth}}
    plane  {-y, 0}
    
    bounded_by {
        box {<-1, -1, 0>, <1, 0, 1>}
    }
    scale <1, .5, 1.2>
    rotate x*45
}

// penguin beak gap (for head)
#declare PBeakG = union {
    cylinder {
        z-z, z*4, .76
        scale <1, .5, 1.2>
        rotate x*45
    }
    scale .6
    translate -y/2 + z*1.9
    texture {m_PMouth}
}

// penguin beak
// h ~= ?, d = 0.72
#declare PBeak = union {
    object {PBeakU}
    object {PBeakD}
    scale .6
    translate -y/2 + z*1.9
}


/* PENGUIN FACE + EYES + BEAK GAP
________________________________________*/


// penguin head sphere
// r = 2
#declare PFace1 = sphere {
    y-y, 2
}

// penguin face area
#declare PFace2 = cylinder {
    y-y, z*2, 2*.75
}

// penguin face + eyes
// r = 2
#declare PFace = union {
    difference {
        object {PFace1} object {PFace2}
        texture {m_PBody1}
    }
    difference {
        intersection {
            object {PFace1} object {PFace2}
        }
        object {PEyes1} object {PEyes2}
        object {PBeakG}
        texture {m_PBody2}
    }
    intersection {
        object {PFace1} object {PEyes1}
        texture {m_PEyes1}
    }
    intersection {
        object {PFace1} object {PEyes2}
        texture {m_PEyes2}
    }
    
    bounded_by {
        object {PFace1}
    }
}


/* PENGUIN HAT
________________________________________*/


// hat elevation
// on sphere at level where r = 1.5
// 2*sin(A) where A is arccos(1.5/2)
#declare PHElev = 1.32288

// submerged lower ring band
// 1.5 : arbitrary radius
// h = 0.6, c. r = 2.1
#declare PHatR = torus {
    1.5+.3, .3
}

// fluffy ball
// r = 0.5
#declare PHatB = sphere {
    y-y, .5
    translate <-3,1,0>
}

// red hat patch coordinates
#declare X1 = 0
#declare X2 = 0
#declare X3 = 0
#declare X4 = -3

#declare Y1 = 0
#declare Y2 = 0
#declare Y3 = 3
#declare Y4 = 1

#declare R1 = 1.8
#declare R2 = 1.8
#declare R3 = 1
#declare R4 = .01

// right/down half + left/up half
#declare Steps = 5
#declare PHatP = union {
    bicubic_patch {
        type 1
        flatness 0
        u_steps Steps
        v_steps Steps
        <X1,Y1,-R1>,        <X2,Y2,-R2>,        <X3,Y3,-R3>,        <X4,Y4,-R4>,
        <X1-R1*4/3,Y1,-R1>, <X2-R2*4/3,Y2,-R2>, <X3,Y3-R3*4/3,-R3>, <X4,Y4-R4*4/3,-R4>,
        <X1-R1*4/3,Y1, R1>, <X2-R2*4/3,Y2, R2>, <X3,Y3-R3*4/3, R3>, <X4,Y4-R4*4/3, R4>,
        <X1,Y1, R1>,        <X2,Y2, R2>,        <X3,Y3, R3>,        <X4,Y4, R4>
    }
    bicubic_patch {
        type 1
        flatness 0
        u_steps Steps
        v_steps Steps
        <X1,Y1,-R1>,        <X2,Y2,-R2>,        <X3,Y3,-R3>,        <X4,Y4,-R4>,
        <X1+R1*4/3,Y1,-R1>, <X2+R2*4/3,Y2,-R2>, <X3,Y3+R3*4/3,-R3>, <X4,Y4+R4*4/3,-R4>,
        <X1+R1*4/3,Y1, R1>, <X2+R2*4/3,Y2, R2>, <X3,Y3+R3*4/3, R3>, <X4,Y4+R4*4/3, R4>,
        <X1,Y1, R1>,        <X2,Y2, R2>,        <X3,Y3, R3>,        <X4,Y4, R4>
    }
}

// penguin hat + lower ring + fluffy ball
// h ~= 3.5 or 2.5 - 0.3 + 1.3
#declare PHat = union {
    object {PHatP texture {m_PHat1}}
    object {PHatR texture {m_PHat2}}
    object {PHatB texture {m_PHat2}}
    
    bounded_by {
        box {<-3.5, -.3, -2.11>, <2.11, 2.2, 2.11>}
    }
    translate y*(PHElev)
}


/* PENGUIN HEAD
________________________________________*/


// penguin head
// h ~= 2.6 (c. h ~= 5.5), r = 0.75
#declare PHead = union {
    object {PFace}
    object {PBeak texture {m_PBeak}}
    object {PHat}
    
    bounded_by {
        box {<-3.5, -2, -2>, <2.11, 3.5, 2.7>}
    }
    scale .5*.75
    translate y*2.9
}


/* PENGUIN BODY
________________________________________*/


// lower body half (under ground)
// h = 1
#declare PBodyD = difference {
    sphere {y-y, 1}
    plane  {-y, 0}
}

// upper body half
// h = 1.5
#declare PBodyU = difference {
    sphere {y-y, 1 scale <1,1.5,1>}
    plane  {y, 0}
}

// belly lower half (under ground)
#declare PBellyD = difference {
    cylinder {z-z, z, 1}
    plane    {-y, 0}
}

// belly upper half
#declare PBellyU = difference {
    cylinder {z-z, z, 1 scale <1,1.5,1>}
    plane    {y, 0}
}

// penguin outer body
// h = 2.5, r = 1
#declare PBody1 = merge {
    object {PBodyD} object {PBodyU}
}

// penguin inner belly
#declare PBody2 = merge {
    object {PBellyD} object {PBellyU}
    scale <.6, .6, 1>
    translate y/2
}

// penguin body
// h = 2.5, r = 1
#declare PBody = union {
    difference {
        object {PBody1} object {PBody2}
        texture {m_PBody1}
    }
    intersection {
        object {PBody1} object {PBody2}
        texture {m_PBody2}
    }
    
    bounded_by {
        sphere {y-y, 1 scale <1,1.5,1>}
    }
    translate y
}


/* PENGUIN WINGS
________________________________________*/


// ding dong surface (trimmed)
// prev r = 0.3849 or sqrt(4/27)
// h = 1, r = 1
#declare DingDong = intersection {
    cubic {<
         0,  0,  0,  1,
         0,  0,  0,  0,
         0,  0,  1,  0,
        -1,  0,  0,  0,
         0,  1,  0,  0
    > sturm}
    sphere {y*5/9, 5/9 texture {None}}
    
    bounded_by {
        sphere {y/2, 1/2}
    }
    scale y + (x+z)*2.59808
}

// submerged vertical wing
// prev h = 1 (1/6 + 5/6), prev r = 1
// h = 2 (1/3 + 5/3), r x = 0.12, r z = 0.3
#declare PWing = object {
    DingDong
    translate -y*5/6
    scale <.12, 1, .3>
    scale 2
}

// penguin left (RHS) wing, lowered
#declare PWingL = object {
    PWing
    rotate -z*35
    rotate -y*5
    translate <-.75, 1.70, -.2>
}

// penguin right (LHS) wing, raised
#declare PWingR = object {
    PWing
    rotate  z*120
    rotate -y*5
    translate < .65, 1.85, -.2>
}


/* PENGUIN TALONS
________________________________________*/


// patch u/v steps
#declare Steps = 5

// penguin talon (flat bottom)
// w = 4, d = 3.5
#declare PTalonD = bicubic_patch {
    type 1
    flatness 0
    u_steps Steps
    v_steps Steps
    <0,0,0>, <1.5,0,0>, <2,0,2.5>, <2,0,2.5>,
    <0,0,0>, <0,0,0>, <0,0,0>, <.5,0,2>,
    <-1.5,0,0>, <0,0,0>, <0,0,0>, <0,0,3.5>,
    <-2,0,2.5>, <-.5,0,2>, <0,0,3.5>, <0,0,3.5>
}

// penguin talon (curved top)
// h ~= 1, w = 4, d = 3.5
#declare PTalonU = bicubic_patch {
    type 1
    flatness 0
    u_steps Steps
    v_steps Steps
    <0,0,0>, <1.5,0,0>, <2,0,2.5>, <2,0,2.5>,
    <0,0,0>, <0,2,0>, <0,2,0>, <.5,0,2>,
    <-1.5,0,0>, <0,2,0>, <0,2,0>, <0,0,3.5>,
    <-2,0,2.5>, <-.5,0,2>, <0,0,3.5>, <0,0,3.5>
    
}

// penguin talon
#declare PTalon = union {
    object {PTalonD} object {PTalonU}
    
    bounded_by {
        box {<-2, 0, 0>, <2, 1, 3.5>}
    }
    scale .3
    rotate y*22
    rotate x*-75
}

// penguin talons
#declare PTalons = union {
    object {PTalon translate z*.85 rotate y*35}
    object {PTalon translate z*.85 rotate y*35 scale -x+y+z}
}


/* PENGUIN
________________________________________*/


// penguin without talons
// h ~= 4.2, w ~= 3.8
#declare Penguin_ = union {
    object {PHead}
    object {PBody}
    object {PWingR texture {m_PBody1}}
    object {PWingL texture {m_PBody1}}
    
    bounded_by {
        box {<-1.7, 0, -1>, <2.1, 4.2, 1>}
    }
}

// penguin with talons (upright)
#declare Penguin = union {
    object {Penguin_}
    object {PTalons texture {m_PTalons}}
}

// penguin with talons (tilted)
#declare PenguinTilted = union {
    object {Penguin_ rotate z*5}
    object {PTalons texture {m_PTalons}}
}
