/* SNOW GLOBE DIMENSIONS
________________________________________*/


// globe radius
#declare SRadius = 3

// globe lower half height
#declare SGlobeLH = SRadius/3

// base height
#declare SBaseH = SRadius/4

// base radius
#declare SBaseR = SRadius*1.08


/* GLASS GLOBE
________________________________________*/


// glass globe
// h = 4 (c. h = 4.75), r = 3
#declare SGlobe = merge {
    difference {
        sphere {y-y, SRadius} plane {y, 0}
        translate y*SGlobeLH
    }
    cylinder {y-y, y*SGlobeLH, SRadius}
    translate y*SBaseH
    
    bounded_by {
        box {
            <-SRadius, SBaseH, -SRadius>,
            < SRadius, SBaseH+SGlobeLH+SRadius, SRadius>
        }
    }
}


/* BASE
________________________________________*/


// 2D ding dong along z
// h = 1, r max = 0.3849 or sqrt(4/27)
#declare DingDongH = intersection {
    cubic {<
         0,  0,  0,  1,
         0,  0,  0,  0,
         0,  0,  1,  0,
        -1,  0,  0,  0,
         0,  0,  0,  0
    >}
    plane {-y, 0 texture {None}}
}

// heart tube from two 2D ding dongs
// prev h ~= 1.43, w ~= 1.5
// h = 1, w ~= 1.05
#declare HeartTube = merge {
    object {DingDongH scale <1,1.5,1> rotate z* 20}
    object {DingDongH scale <1,1.5,1> rotate z*-20}
    
    bounded_by {
        box {<-.75, 0, -1>, <.75, 1.43, 1>}
    }
    scale 1/1.43
}

// resized heart tube vertically centered in base
// h = 0.75*0.55
#declare HeartTube_ = object {
    HeartTube
    translate -y/2
    scale <SBaseH*.55, SBaseH*.55, SBaseR>
    translate y*SBaseH/2
}

// base shape
#declare SBase_ = cylinder {
    y-y, y*SBaseH, SBaseR
}

// base with heart hole + heart
#declare SBase1 = difference {
    object {SBase_} object {HeartTube_}
    texture {m_SBase1}
}
#declare SBase2 = intersection {
    object {SBase_} object {HeartTube_}
    texture {m_SBase2}
}

// base
// h = 0.75, r = 3.24
#declare SBase = merge {
    object {SBase1} object {SBase2}
    
    bounded_by {
        box {<-SBaseR, 0, -SBaseR>, <SBaseR, SBaseH, SBaseR>}
    }
}


/* SNOW FLOOR
________________________________________*/


// snow floor made of blobs
// h ~= 0.5, r = 3.24
#declare SFloor = intersection {
    blob {
        threshold 5/9
        component 1, 1, <   0,    0,    0>
        component 1, 1, <-1.2,  -.4,    0>
        component 1, 1, <   1,  -.2,   .2>
        component 1, 1, <   0,  -.8,   .8>
        component 1, 1, < -.6,   .3,  -.6>
        component 1, 1, <  .8,   .4,  -.6>
        component 1, .5, < -.4,  -.4,   .4>
        component 1, .5, <  .5,   -1,   .5>
        component 1, .5, <   0,   .6,  -.8>
        translate y/2
        rotate z*15
        scale <SRadius*2/1.5, .3, SRadius*2/1.5>
        translate y*SBaseH
        rotate y*-15
    }
    object {SGlobe scale <(SRadius-.001)/SRadius, 1, (SRadius-.001)/SRadius>}
    
    bounded_by {
        box {<-SRadius, SBaseH, -SRadius>, <SRadius, SBaseH+.5, SRadius>}
    }
}


/* SNOW FALL
________________________________________*/


// falling snowflakes inside globe
#declare SFall = difference {
    sphere {
        y*(SGlobeLH+SBaseH), SRadius-.001
    }
    plane {y, SBaseH}
    no_shadow
}


/* SNOW GLOBE
________________________________________*/


// snow globe
// h = 4.75, r = 3.24
#declare SnowGlobe = union {
    object {SGlobe texture {m_SGlobe}}
    object {SBase}
    object {SFloor texture {m_SFloor}}
    object {SFall texture {m_SFall}}
    
    bounded_by {
        box {
            <-SBaseR, 0, -SBaseR>,
            <SBaseR, SBaseH+SGlobeLH+SRadius, SBaseR>
        }
    }
}
